-----------------------------------------------------------------------
-- HEIG-VD, Haute Ecole d'Ingenerie et de Gestion du Canton de Vaud
-- Institut REDS
--
-- Composant    : bcd_adder_tb
-- Description  : Banc de test pour un additionneur BCD.
--                Un paramètre générique permet de définir le nombre de digits.
--                Un paramètre générique permet de forcer des erreurs
--                du DUT.
--                Il n'y a pas de vérification automatique implémentée
-- Auteur       : Yann Thoma
-- Date         : 01.03.2017
-- Version      : 1.0
--
-- Modification : -
--
-----------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use IEEE.math_real.all;
use work.bcd_pkg.all;

entity bcd_adder_tb is
generic (
    ERRNO   : integer := 0;
    NDIGITS : integer := 4
);
end bcd_adder_tb;

architecture testbench of bcd_adder_tb is

    component bcd_adder is
    generic (
        ERRNO   : integer := 0;
        NDIGITS : integer := 4
    );
    port (
        input0_i  : in  bcd_number(NDIGITS - 1 downto 0);
        input1_i  : in  bcd_number(NDIGITS - 1 downto 0);
        result_o  : out bcd_number(NDIGITS downto 0);
        hamming_o : out std_logic_vector(integer(ceil(log2(real(4*NDIGITS)))) downto 0)
    );
    end component;


    signal input0_sti  : bcd_number(NDIGITS - 1 downto 0);
    signal input1_sti  : bcd_number(NDIGITS - 1 downto 0);
    signal result_obs  : bcd_number(NDIGITS downto 0);
    signal hamming_obs : std_logic_vector(integer(ceil(log2(real(4*NDIGITS)))) downto 0);

begin

    dut: bcd_adder
    generic map (
        ERRNO => ERRNO,
        NDIGITS => NDIGITS
    )
    port map(
        input0_i => input0_sti,
        input1_i => input1_sti,
        result_o => result_obs,
        hamming_o => hamming_obs
    );

    process is
    begin
        report "Starting simulation with ERRNO = " & integer'image(ERRNO);
        input0_sti <= (others => (others => '0'));
        input1_sti <= (others => (others => '0'));

        wait for 10 ns;

        input0_sti(0)(0) <= '1';
        input1_sti(0)(1) <= '1';
        wait for 10 ns;

        wait;

    end process;

end testbench;
